/*
 * Nombre del proyecto: Proyecto Arkanoid
 *
 * Nombre del fichero: errores.c
 *
 * Nombre de los autores: Grzegorz Adam Hankiewicz y Carlos Pereda Moreno
 *
 * Fecha: 18 de Enero del 2000
 *
 * Descripcin: Cdigo fuente que implementa el sistema de errores del
 * programa, facilitando una funcin genrica tipo eprintf que puede
 * ser llamada desde cualquier parte del programa y que permitir mostrar
 * x mensajes de error en cualquier sistema, bajo cualquier condicin,
 * al finalizar el programa.
 */

/* CABECERAS GLOBALES */
#ifdef MSS
#include <mss.h>
#endif MSS
#include <stdlib.h>
#include <allegro.h>

/* CABECERAS DE FUNCIONES EXTERNAS */
#include "general.h"
#include "init.h"

/* CABECERA PROPIA */
#define DECLARA_VARIABLES_ERRORES_C
#include "errores.h"


/**[txh]********************************************************************

  Description: Inicializa las variables del sistema de errores. El sistema
  de errores se basa en lo siguiente: cuando se quiere dar un mensaje normal
  al usuario, se muestra en pantalla o se usan las rutinas de logging. Sin
  embargo, cuando hay un error grave, es posible que adems el sistema no
  est en un estado adecuado como para mostrarlo. Por eso, se almacena el
  mensaje de error en un array esttico, y al finalizar el programa se
  muestran todos de golpe, en orden de ocurrencia.
  @*
  Se usa un array esttico, ya que as no puede dar fallo el malloc para
  reservar memoria para la cadena del mensaje (cosa que s puede pasar con
  el sistema de logging), y para que la inicializacin del propio sistema
  de errores no pueda fallar.

***************************************************************************/


void     inicializa_mensajes_de_error (void)
{
   int f;

   /* Primero 'cortamos' las cadenas de error poniendo el carcter 0 */
   for (f = NUM_CADENAS_ERROR-1; f >= 0; f--)
      mensajes_de_error[f][0] = 0;

   /* Y ponemos a cero el contador. */
   num_mensajes_error = 0;
}


/**[txh]********************************************************************

  Description: Rutina de parmetros variables tipo printf, que almacena en
  un array esttico un mensaje que ser visualizado al salir del programa.
  Primero convierte la lista de parmetros en una cadena (buf), para luego
  almacenar esa cadena buf en el array, en la posicion adecuada.
  @*
  NOTA: si el programa llena la tabla de errores, se producira un desborde
  con el siguiente, por lo que el programa aborta inmediatamente y sale con
  exit(valor_error).

  Example:
  @<pre>
  if (retorno != 0)
  {
    @x{eprintf} ("Error al volver de la rutina, retorno %d", retorno);
    return retorno;
  }
  @</pre>

***************************************************************************/

void     eprintf (char *msg, ...)
{
   char buf[STR];

   /* Transformacin de parmetros en cadena simple */
   va_list ap;
   va_start (ap, msg);
   uvsprintf (buf, msg, ap);
   va_end (ap);

   /* Nos aseguramos de que la cadena esta vaca e imprimimos el mensaje */
   mensajes_de_error[num_mensajes_error][0] = 0;

   /* A continuacin se usa la funcin ustrncat de Allegro en vez de la
    * funcin estndar strncat, porque la de Allegro es 100% portable, y
    * respeta los caracteres unicode, aparte de tener proteccin de desborde.
    */

   ustrncat (mensajes_de_error[num_mensajes_error], buf, SMALL_STR-1);
   /* Comprobamos si hay que aadir carcter de retorno. */
   if (ugetat (mensajes_de_error[num_mensajes_error], -1) != '\n')
      uinsert (mensajes_de_error[num_mensajes_error], -0, '\n');
      
   num_mensajes_error++;
   if (num_mensajes_error == NUM_CADENAS_ERROR-1)
   {
      /* Pequea recursin en la funcin. Sabemos que slo se hace una vez
       * y que adems resulta inofensiva por haber reservado una cadena
       * para notificar este desborde en la propia lista esttica. Por cierto:
       * ahora si hay que explicitar los \n.
       */
      eprintf ("\neprintf: Desborde, %d mensajes pendientes en la cola de errores.",
               num_mensajes_error);
      /* Antes de llamar a exit nos aseguramos de que "limpiamos" todos los
       * mdulos que hemos inicializado. */
      finaliza_programa();
      exit (10);
   }
}


/**[txh]********************************************************************

  Description: Esta funcin muestra los mensajes de error que se han ido
  acumulando, en orden de ocurrencia.
  @*
  Atencin: se debe usar slo cuando ya se ha salido del modo grfico,
  es decir, desde @x{finaliza_programa}.

***************************************************************************/

void     muestra_mensajes_de_error (void)
{
   int f;

   if (num_mensajes_error < 1) /* Si no hay errores, salimos */
      return ;

   for (f = 0; f < num_mensajes_error; f++)
      allegro_message ("%s\n", mensajes_de_error[f]);
   allegro_message (" \n");
}

